@@ -9,7 +9,7 @@
  */
 
 #include <sm/gen.h>
-SM_RCSID("@(#)$Id: listener.c,v 8.85.2.12 2003/08/04 18:47:29 ca Exp $")
+SM_RCSID("@(#)$Id: listener.c,v 8.109 2004/02/04 22:55:59 ca Exp $")
 
 /*
 **  listener.c -- threaded network listener
@@ -18,6 +18,9 @@ SM_RCSID("@(#)$Id: listener.c,v 8.85.2.12 2003/08/04 18:47:29 ca Exp $")
 #include "libmilter.h"
 #include <sm/errstring.h>
 
+#include <sys/types.h>
+#include <sys/stat.h>
+
 
 # if NETINET || NETINET6
 #  include <arpa/inet.h>
@@ -28,26 +31,29 @@ static int L_family;
 static SOCKADDR_LEN_T L_socksize;
 static socket_t listenfd = INVALID_SOCKET;
 
-static socket_t mi_milteropen __P((char *, int, char *));
+static socket_t mi_milteropen __P((char *, int, bool, char *));
 
 /*
 **  MI_OPENSOCKET -- create the socket where this filter and the MTA will meet
 **
-**  	Parameters:
+**	Parameters:
 **		conn -- connection description
 **		backlog -- listen backlog
-**  		dbg -- debug level
+**		dbg -- debug level
+**		rmsocket -- if true, try to unlink() the socket first
+**			(UNIX domain sockets only)
 **		smfi -- filter structure to use
 **
-**  	Return value:
-**  		MI_SUCCESS/MI_FAILURE
+**	Return value:
+**		MI_SUCCESS/MI_FAILURE
 */
 
 int
-mi_opensocket(conn, backlog, dbg, smfi)
+mi_opensocket(conn, backlog, dbg, rmsocket, smfi)
 	char *conn;
 	int backlog;
 	int dbg;
+	bool rmsocket;
 	smfiDesc_ptr smfi;
 {
 	if (smfi == NULL || conn == NULL)
@@ -64,7 +70,7 @@ mi_opensocket(conn, backlog, dbg, smfi)
 	}
 	(void) smutex_init(&L_Mutex);
 	(void) smutex_lock(&L_Mutex);
-	listenfd = mi_milteropen(conn, backlog, smfi->xxfi_name);
+	listenfd = mi_milteropen(conn, backlog, rmsocket, smfi->xxfi_name);
 	if (!ValidSocket(listenfd))
 	{
 		smi_log(SMI_LOG_FATAL,
@@ -73,7 +79,7 @@ mi_opensocket(conn, backlog, dbg, smfi)
 		(void) smutex_unlock(&L_Mutex);
 		return MI_FAILURE;
 	}
-#if !_FFR_USE_POLL
+#if !SM_CONF_POLL
 	if (!SM_FD_OK_SELECT(listenfd))
 	{
 		smi_log(SMI_LOG_ERR, "%s: fd %d is larger than FD_SETSIZE %d",
@@ -81,7 +87,7 @@ mi_opensocket(conn, backlog, dbg, smfi)
 		(void) smutex_unlock(&L_Mutex);
 		return MI_FAILURE;
 	}
-#endif /* !_FFR_USE_POLL */
+#endif /* !SM_CONF_POLL */
 	return MI_SUCCESS;
 }
 
@@ -91,6 +97,8 @@ mi_opensocket(conn, backlog, dbg, smfi)
 **	Parameters:
 **		conn -- connection description
 **		backlog -- listen backlog
+**		rmsocket -- if true, try to unlink() the socket first
+**			(UNIX domain sockets only)
 **		name -- name for logging
 **
 **	Returns:
@@ -105,9 +113,10 @@ static char	*sockpath = NULL;
 #endif /* NETUNIX */
 
 static socket_t
-mi_milteropen(conn, backlog, name)
+mi_milteropen(conn, backlog, rmsocket, name)
 	char *conn;
 	int backlog;
+	bool rmsocket;
 	char *name;
 {
 	socket_t sock;
@@ -457,6 +466,41 @@ mi_milteropen(conn, backlog, name)
 		return INVALID_SOCKET;
 	}
 
+#if NETUNIX
+	if (addr.sa.sa_family == AF_UNIX && rmsocket)
+	{
+		struct stat s;
+
+		if (stat(colon, &s) != 0)
+		{
+			if (errno != ENOENT)
+			{
+				smi_log(SMI_LOG_ERR,
+					"%s: Unable to stat() %s: %s",
+					name, colon, sm_errstring(errno));
+				(void) closesocket(sock);
+				return INVALID_SOCKET;
+			}
+		}
+		else if (!S_ISSOCK(s.st_mode))
+		{
+			smi_log(SMI_LOG_ERR,
+				"%s: %s is not a UNIX domain socket",
+				name, colon);
+			(void) closesocket(sock);
+			return INVALID_SOCKET;
+		}
+		else if (unlink(colon) != 0)
+		{
+			smi_log(SMI_LOG_ERR,
+				"%s: Unable to remove %s: %s",
+				name, colon, sm_errstring(errno));
+			(void) closesocket(sock);
+			return INVALID_SOCKET;
+		}
+	}
+#endif /* NETUNIX */
+
 	if (bind(sock, &addr.sa, L_socksize) < 0)
 	{
 		smi_log(SMI_LOG_ERR,
@@ -541,9 +585,7 @@ mi_closener()
 		struct stat fileinfo;
 
 		removable = sockpath != NULL &&
-#if _FFR_MILTER_ROOT_UNSAFE
 			    geteuid() != 0 &&
-#endif /* _FFR_MILTER_ROOT_UNSAFE */
 			    fstat(listenfd, &sockinfo) == 0 &&
 			    (S_ISFIFO(sockinfo.st_mode)
 # ifdef S_ISSOCK
@@ -593,7 +635,7 @@ mi_closener()
 **		dbg -- debug level
 **		smfi -- filter structure to use
 **		timeout -- timeout for reads/writes
-**  		backlog -- listen queue backlog size
+**		backlog -- listen queue backlog size
 **
 **	Returns:
 **		MI_SUCCESS -- Exited normally
@@ -658,6 +700,9 @@ mi_listener(conn, dbg, smfi, timeout, backlog)
 	int backlog;
 {
 	socket_t connfd = INVALID_SOCKET;
+#if _FFR_DUP_FD
+	socket_t dupfd = INVALID_SOCKET;
+#endif /* _FFR_DUP_FD */
 	int sockopt = 1;
 	int r, mistop;
 	int ret = MI_SUCCESS;
@@ -673,7 +718,7 @@ mi_listener(conn, dbg, smfi, timeout, backlog)
 	FD_RD_VAR(rds, excs);
 	struct timeval chktime;
 
-	if (mi_opensocket(conn, backlog, dbg, smfi) == MI_FAILURE)
+	if (mi_opensocket(conn, backlog, dbg, false, smfi) == MI_FAILURE)
 		return MI_FAILURE;
 
 	clilen = L_socksize;
@@ -756,7 +801,7 @@ mi_listener(conn, dbg, smfi, timeout, backlog)
 			save_errno = EINVAL;
 		}
 
-#if !_FFR_USE_POLL
+#if !SM_CONF_POLL
 		/* check if acceptable for select() */
 		if (ValidSocket(connfd) && !SM_FD_OK_SELECT(connfd))
 		{
@@ -764,11 +809,36 @@ mi_listener(conn, dbg, smfi, timeout, backlog)
 			connfd = INVALID_SOCKET;
 			save_errno = ERANGE;
 		}
-#endif /* !_FFR_USE_POLL */
+#endif /* !SM_CONF_POLL */
 
 		if (!ValidSocket(connfd))
 		{
-			if (save_errno == EINTR)
+			if (save_errno == EINTR
+#ifdef EAGAIN
+			    || save_errno == EAGAIN
+#endif /* EAGAIN */
+#ifdef ECONNABORTED
+			    || save_errno == ECONNABORTED
+#endif /* ECONNABORTED */
+#ifdef EMFILE
+			    || save_errno == EMFILE
+#endif /* EMFILE */
+#ifdef ENFILE
+			    || save_errno == ENFILE
+#endif /* ENFILE */
+#ifdef ENOBUFS
+			    || save_errno == ENOBUFS
+#endif /* ENOBUFS */
+#ifdef ENOMEM
+			    || save_errno == ENOMEM
+#endif /* ENOMEM */
+#ifdef ENOSR
+			    || save_errno == ENOSR
+#endif /* ENOSR */
+#ifdef EWOULDBLOCK
+			    || save_errno == EWOULDBLOCK
+#endif /* EWOULDBLOCK */
+			   )
 				continue;
 			acnt++;
 			smi_log(SMI_LOG_ERR,
@@ -784,6 +854,19 @@ mi_listener(conn, dbg, smfi, timeout, backlog)
 			continue;
 		}
 		acnt = 0;	/* reset error counter for accept() */
+#if _FFR_DUP_FD
+		dupfd = fcntl(connfd, F_DUPFD, 256);
+		if (ValidSocket(dupfd)
+# if !SM_CONF_POLL
+		    && SM_FD_OK_SELECT(dupfd)
+# endif /* !SM_CONF_POLL */
+		   )
+		{
+			close(connfd);
+			connfd = dupfd;
+			dupfd = INVALID_SOCKET;
+		}
+#endif /* _FFR_DUP_FD */
 
 		if (setsockopt(connfd, SOL_SOCKET, SO_KEEPALIVE,
 				(void *) &sockopt, sizeof sockopt) < 0)
