@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1998-2002 Sendmail, Inc. and its suppliers.
+ * Copyright (c) 1998-2004 Sendmail, Inc. and its suppliers.
  *	All rights reserved.
  * Copyright (c) 1983, 1995-1997 Eric P. Allman.  All rights reserved.
  * Copyright (c) 1988, 1993
@@ -13,10 +13,9 @@
 
 #include <sendmail.h>
 
-SM_RCSID("@(#)$Id: collect.c,v 8.242.2.4 2003/03/28 17:34:39 ca Exp $")
+SM_RCSID("@(#)$Id: collect.c,v 8.254 2004/04/05 18:41:38 ca Exp $")
 
 static void	collecttimeout __P((time_t));
-static void	dferror __P((SM_FILE_T *volatile, char *, ENVELOPE *));
 static void	eatfrom __P((char *volatile, ENVELOPE *));
 static void	collect_doheader __P((ENVELOPE *));
 static SM_FILE_T *collect_dfopen __P((ENVELOPE *));
@@ -217,7 +216,7 @@ collect_dfopen(e)
 		syserr("@Cannot create %s", dfname);
 		e->e_flags |= EF_NO_BODY_RETN;
 		flush_errors(true);
-		finis(true, true, ExitStat);
+		finis(false, true, ExitStat);
 		/* NOTREACHED */
 	}
 	dfd = sm_io_getinfo(df, SM_IO_WHAT_FD, NULL);
@@ -247,6 +246,7 @@ collect_dfopen(e)
 **			end of message.
 **		hdrp -- the location to stash the header.
 **		e -- the current envelope.
+**		rsetsize -- reset e_msgsize?
 **
 **	Returns:
 **		none.
@@ -281,11 +281,12 @@ static SM_EVENT	*volatile CollectTimeout = NULL;
 #define MS_DISCARD	3	/* discarding rest of message */
 
 void
-collect(fp, smtpmode, hdrp, e)
+collect(fp, smtpmode, hdrp, e, rsetsize)
 	SM_FILE_T *fp;
 	bool smtpmode;
 	HDR **hdrp;
 	register ENVELOPE *e;
+	bool rsetsize;
 {
 	register SM_FILE_T *volatile df;
 	volatile bool ignrdot;
@@ -366,7 +367,8 @@ collect(fp, smtpmode, hdrp, e)
 		CollectTimeout = sm_setevent(dbto, collecttimeout, dbto);
 	}
 
-	e->e_msgsize = 0;
+	if (rsetsize)
+		e->e_msgsize = 0;
 	for (;;)
 	{
 		if (tTd(30, 35))
@@ -721,7 +723,9 @@ readerr:
 		finis(true, true, ExitStat);
 		/* NOTREACHED */
 	}
-	else if (SuperSafe != SAFE_REALLY)
+	else if (SuperSafe == SAFE_NO ||
+		 SuperSafe == SAFE_INTERACTIVE ||
+		 (SuperSafe == SAFE_REALLY_POSTMILTER && smtpmode))
 	{
 		/* skip next few clauses */
 		/* EMPTY */
@@ -740,7 +744,7 @@ readerr:
 			if (stat(dfile, &st) < 0)
 				st.st_size = -1;
 			errno = EEXIST;
-			syserr("@collect: bfcommit(%s): already on disk, size = %ld",
+			syserr("@collect: bfcommit(%s): already on disk, size=%ld",
 			       dfile, (long) st.st_size);
 			dfd = sm_io_getinfo(df, SM_IO_WHAT_FD, NULL);
 			if (dfd >= 0)
@@ -751,8 +755,14 @@ readerr:
 		flush_errors(true);
 		finis(save_errno != EEXIST, true, ExitStat);
 	}
-	else if ((afd = sm_io_getinfo(df, SM_IO_WHAT_FD, NULL)) >= 0 &&
-		 fsync(afd) < 0)
+	else if ((afd = sm_io_getinfo(df, SM_IO_WHAT_FD, NULL)) < 0)
+	{
+		dferror(df, "sm_io_getinfo", e);
+		flush_errors(true);
+		finis(true, true, ExitStat);
+		/* NOTREACHED */
+	}
+	else if (fsync(afd) < 0)
 	{
 		dferror(df, "fsync", e);
 		flush_errors(true);
@@ -870,7 +880,7 @@ readerr:
 	{
 		char *dfname = queuename(e, DATAFL_LETTER);
 		if ((e->e_dfp = sm_io_open(SmFtStdio, SM_TIME_DEFAULT, dfname,
-					   SM_IO_RDONLY, NULL)) == NULL)
+					   SM_IO_RDONLY_B, NULL)) == NULL)
 		{
 			/* we haven't acked receipt yet, so just chuck this */
 			syserr("@Cannot reopen %s", dfname);
@@ -883,7 +893,18 @@ readerr:
 
 	/* collect statistics */
 	if (OpMode != MD_VERIFY)
+	{
+		/*
+		**  Recalculate e_msgpriority, it is done at in eatheader()
+		**  which is called (in 8.12) after the header is collected,
+		**  hence e_msgsize is (most likely) incorrect.
+		*/
+
+		e->e_msgpriority = e->e_msgsize
+				 - e->e_class * WkClassFact
+				 + e->e_nrcpts * WkRecipFact;
 		markstats(e, (ADDRESS *) NULL, STATS_NORMAL);
+	}
 }
 
 static void
@@ -940,7 +961,7 @@ collecttimeout(timeout)
 **		Arranges for following output to go elsewhere.
 */
 
-static void
+void
 dferror(df, msg, e)
 	SM_FILE_T *volatile df;
 	char *msg;
@@ -971,7 +992,7 @@ dferror(df, msg, e)
 		    < 0)
 		  st.st_size = 0;
 		(void) sm_io_reopen(SmFtStdio, SM_TIME_DEFAULT, dfname,
-				    SM_IO_WRONLY, NULL, df);
+				    SM_IO_WRONLY_B, NULL, df);
 		if (st.st_size <= 0)
 			(void) sm_io_fprintf(df, SM_TIME_DEFAULT,
 				"\n*** Mail could not be accepted");
@@ -1019,6 +1040,7 @@ dferror(df, msg, e)
 **
 **	Parameters:
 **		fm -- the from line.
+**		e -- envelope
 **
 **	Returns:
 **		none.
