@@ -13,7 +13,7 @@
 
 #include <sendmail.h>
 
-SM_RCSID("@(#)$Id: mci.c,v 8.205.2.3 2003/01/07 03:56:19 ca Exp $")
+SM_RCSID("@(#)$Id: mci.c,v 8.211 2003/03/31 17:35:50 ca Exp $")
 
 #if NETINET || NETINET6
 # include <arpa/inet.h>
@@ -487,6 +487,7 @@ mci_setstat(mci, xstat, dstat, rstat)
 **  MCI_DUMP -- dump the contents of an MCI structure.
 **
 **	Parameters:
+**		fp -- output file pointer
 **		mci -- the MCI structure to dump.
 **
 **	Returns:
@@ -529,7 +530,8 @@ static struct mcifbits	MciFlags[] =
 };
 
 void
-mci_dump(mci, logit)
+mci_dump(fp, mci, logit)
+	SM_FILE_T *fp;
 	register MCI *mci;
 	bool logit;
 {
@@ -548,11 +550,21 @@ mci_dump(mci, logit)
 	}
 	(void) sm_snprintf(p, SPACELEFT(buf, p), "flags=%lx", mci->mci_flags);
 	p += strlen(p);
+
+	/*
+	**  The following check is just for paranoia.  It protects the
+	**  assignment in the if() clause. If there's not some minimum
+	**  amount of space we can stop right now. The check will not
+	**  trigger as long as sizeof(buf)=4000.
+	*/
+
+	if (p >= buf + sizeof(buf) - 4)
+		goto printit;
 	if (mci->mci_flags != 0)
 	{
 		struct mcifbits *f;
 
-		*p++ = '<';
+		*p++ = '<';	/* protected above */
 		for (f = MciFlags; f->mcif_bit != 0; f++)
 		{
 			if (!bitset(f->mcif_bit, mci->mci_flags))
@@ -587,12 +599,13 @@ printit:
 	if (logit)
 		sm_syslog(LOG_DEBUG, CurEnv->e_id, "%.1000s", buf);
 	else
-		(void) sm_io_fprintf(smioout, SM_TIME_DEFAULT, "%s\n", buf);
+		(void) sm_io_fprintf(fp, SM_TIME_DEFAULT, "%s\n", buf);
 }
 /*
 **  MCI_DUMP_ALL -- print the entire MCI cache
 **
 **	Parameters:
+**		fp -- output file pointer
 **		logit -- if set, log the result instead of printing
 **			to stdout.
 **
@@ -601,7 +614,8 @@ printit:
 */
 
 void
-mci_dump_all(logit)
+mci_dump_all(fp, logit)
+	SM_FILE_T *fp;
 	bool logit;
 {
 	register int i;
@@ -610,7 +624,7 @@ mci_dump_all(logit)
 		return;
 
 	for (i = 0; i < MaxMciCache; i++)
-		mci_dump(MciCache[i], logit);
+		mci_dump(fp, MciCache[i], logit);
 }
 /*
 **  MCI_LOCK_HOST -- Lock host while sending.
@@ -915,7 +929,7 @@ mci_read_persistent(fp, mci)
 
 		  case '.':		/* end of file */
 			if (tTd(56, 93))
-				mci_dump(mci, false);
+				mci_dump(sm_debug_file(), mci, false);
 			return 0;
 
 		  default:
@@ -1152,7 +1166,7 @@ mci_traverse_persistent(action, pathname)
 			if (hostptr != host)
 				*(hostptr++) = '.';
 			start = end;
-			while (*(start - 1) != '/')
+			while (start > pathname && *(start - 1) != '/')
 				start--;
 
 			if (*end == '.')
@@ -1162,7 +1176,7 @@ mci_traverse_persistent(action, pathname)
 				*(hostptr++) = *scan;
 
 			end = start - 2;
-		} while (*end == '.');
+		} while (end > pathname && *end == '.');
 
 		*hostptr = '\0';
 
@@ -1352,7 +1366,7 @@ mci_purge_persistent(pathname, hostname)
 /*
 **  MCI_GENERATE_PERSISTENT_PATH -- generate path from hostname
 **
-**	Given `host', convert from a.b.c to $QueueDir/.hoststat/c./b./a,
+**	Given `host', convert from a.b.c to $HostStatDir/c./b./a,
 **	putting the result into `path'.  if `createflag' is set, intervening
 **	directories will be created as needed.
 **
