@@ -13,7 +13,7 @@
 
 #include <sendmail.h>
 
-SM_RCSID("@(#)$Id: recipient.c,v 8.335 2003/10/06 20:37:56 ca Exp $")
+SM_RCSID("@(#)$Id: recipient.c,v 8.336 2004/07/23 20:45:02 gshapiro Exp $")
 
 static void	includetimeout __P((void));
 static ADDRESS	*self_reference __P((ADDRESS *));
@@ -1309,9 +1309,20 @@ writable(filename, ctladdr, flags)
 	}
 	else if (FileMailer != NULL && !bitset(SFF_ROOTOK, flags))
 	{
-		euid = FileMailer->m_uid;
-		egid = FileMailer->m_gid;
-		user = NULL;
+		if (FileMailer->m_uid == NO_UID)
+		{
+			euid = DefUid;
+			user = DefUser;
+		}
+		else
+		{
+			euid = FileMailer->m_uid;
+			user = NULL;
+		}
+		if (FileMailer->m_gid == NO_GID)
+			egid = DefGid;
+		else
+			egid = FileMailer->m_gid;
 	}
 	else
 	{
