@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1998-2002 Sendmail, Inc. and its suppliers.
+ * Copyright (c) 1998-2003 Sendmail, Inc. and its suppliers.
  *	All rights reserved.
  * Copyright (c) 1983, 1995-1997 Eric P. Allman.  All rights reserved.
  * Copyright (c) 1988, 1993
@@ -13,7 +13,7 @@
 
 #include <sendmail.h>
 
-SM_RCSID("@(#)$Id: savemail.c,v 8.299.2.1 2002/10/23 15:08:47 ca Exp $")
+SM_RCSID("@(#)$Id: savemail.c,v 8.303 2004/01/14 02:56:51 ca Exp $")
 
 static void	errbody __P((MCI *, ENVELOPE *, char *));
 static bool	pruneroute __P((char *));
@@ -74,7 +74,7 @@ savemail(e, sendbody)
 		sm_dprintf("\nsavemail, errormode = %c, id = %s, ExitStat = %d\n  e_from=",
 			e->e_errormode, e->e_id == NULL ? "NONE" : e->e_id,
 			ExitStat);
-		printaddr(&e->e_from, false);
+		printaddr(sm_debug_file(), &e->e_from, false);
 	}
 
 	if (e->e_id == NULL)
@@ -178,7 +178,12 @@ savemail(e, sendbody)
 			**  then write the error messages back to hir (sic).
 			*/
 
+#if USE_TTYPATH
 			p = ttypath();
+#else /* USE_TTYPATH */
+			p = NULL;
+#endif /* USE_TTYPATH */
+
 			if (p == NULL || sm_io_reopen(SmFtStdio,
 						      SM_TIME_DEFAULT,
 						      p, SM_IO_WRONLY, NULL,
@@ -518,11 +523,11 @@ returntosender(msg, returnq, flags, e)
 	{
 		sm_dprintf("\n*** Return To Sender: msg=\"%s\", depth=%d, e=%p, returnq=",
 			msg, returndepth, e);
-		printaddr(returnq, true);
+		printaddr(sm_debug_file(), returnq, true);
 		if (tTd(6, 20))
 		{
 			sm_dprintf("Sendq=");
-			printaddr(e->e_sendqueue, true);
+			printaddr(sm_debug_file(), e->e_sendqueue, true);
 		}
 	}
 
